@@ -14,7 +14,10 @@
 """Test configuration machinery.
 """
 import unittest
+import sys
 
+# pylint:disable=inherit-non-class,protected-access
+# pylint:disable=attribute-defined-outside-init, arguments-differ
 
 class ConfigurationContextTests(unittest.TestCase):
 
@@ -25,6 +28,13 @@ class ConfigurationContextTests(unittest.TestCase):
     def _makeOne(self, *args, **kw):
         return self._getTargetClass()(*args, **kw)
 
+    def tearDown(self):
+        for name in ('zope.configuration.tests.victim',
+                     'zope.configuration.tests.bad'
+                     'zope.configuration.tests.unicode_all',
+                     'zope.configuration.tests.unicode_all.__future__'):
+            sys.modules.pop(name, None)
+
     def test_resolve_empty(self):
         c = self._makeOne()
         with self.assertRaises(ValueError):
@@ -64,7 +74,7 @@ class ConfigurationContextTests(unittest.TestCase):
 
     def test_resolve_relative_miss_w_package_too_many_dots(self):
         class FauxPackage(object):
-            pass
+            __name__ = None
         from zope.configuration.exceptions import ConfigurationError
         c = self._makeOne()
         package = c.package = FauxPackage()
@@ -91,28 +101,27 @@ class ConfigurationContextTests(unittest.TestCase):
     def test_resolve_bad_sub_last_import(self):
         #Import error caused by a bad sub import inside the referenced
         #dotted name. Here we keep the standard traceback.
-        import sys
         c = self._makeOne()
         with self.assertRaises(ImportError):
             c.resolve('zope.configuration.tests.victim')
-        #Cleanup:
-        for name in ('zope.configuration.tests.victim',
-                     'zope.configuration.tests.bad'):
-           if name in sys.modules:
-               del sys.modules[name]
 
     def test_resolve_bad_sub_import(self):
         #Import error caused by a bad sub import inside part of the referenced
         #dotted name. Here we keep the standard traceback.
-        import sys
         c = self._makeOne()
         with self.assertRaises(ImportError):
             c.resolve('zope.configuration.tests.victim.nosuch')
-        #Cleanup:
-        for name in ('zope.configuration.tests.victim',
-                     'zope.configuration.tests.bad'):
-           if name in sys.modules:
-               del sys.modules[name]
+
+    def test_resolve_unicode_all(self):
+        # If a package's __init__.py is in the process of being ported from
+        # Python 2 to Python 3 using unicode_literals, then it can end up
+        # with unicode items in __all__, which breaks star imports from that
+        # package; but we tolerate this.
+        c = self._makeOne()
+        self.assertEqual(
+            c.resolve('zope.configuration.tests.unicode_all').foo, 'sentinel')
+        self.assertEqual(
+            c.resolve('zope.configuration.tests.unicode_all.foo'), 'sentinel')
 
     def test_path_w_absolute_filename(self):
         import os
@@ -154,6 +163,31 @@ class ConfigurationContextTests(unittest.TestCase):
         c.package = stub()
         self.assertTrue(os.path.isabs(c.path('y/z')))
 
+    def test_path_expand_filenames(self):
+        import os
+        c = self._makeOne()
+        os.environ['path-test'] = '42'
+        try:
+            path = c.path(os.path.join(os.sep, '${path-test}'))
+            self.assertEqual(path, os.path.join(os.sep, '42'))
+        finally:
+            del os.environ['path-test']
+
+    def test_path_expand_user(self):
+        import os
+        c = self._makeOne()
+        old_home = os.environ.get('HOME')
+        # HOME should be absolute
+        os.environ['HOME'] = os.path.join(os.sep, 'HOME')
+        try:
+            path = c.path('~')
+            self.assertEqual(path, os.path.join(os.sep, 'HOME'))
+        finally: # pragma: no cover
+            if old_home:
+                os.environ['HOME'] = old_home
+            else:
+                del os.environ['HOME']
+
     def test_checkDuplicate_miss(self):
         import os
         c = self._makeOne()
@@ -221,7 +255,7 @@ class ConfigurationContextTests(unittest.TestCase):
         INCLUDE_PATH = ('p', 'q/r')
         INFO = 'INFO'
         def _callable():
-            pass
+            raise AssertionError("should not be called")
         c = self._makeOne()
         c.actions = [] # normally provided by subclass
         c.action(DISCRIMINATOR,
@@ -249,7 +283,7 @@ class ConfigurationContextTests(unittest.TestCase):
         INCLUDE_PATH = ('p', 'q/r')
         INFO = 'INFO'
         def _callable():
-            pass
+            raise AssertionError("should not be called")
         c = self._makeOne()
         c.actions = [] # normally provided by subclass
         c.action(DISCRIMINATOR,
@@ -310,7 +344,7 @@ class ConfigurationAdapterRegistryTests(unittest.TestCase):
         NS = 'http://namespace.example.com/'
         NAME = 'testing'
         def _factory():
-            pass
+            raise AssertionError("should not be called")
         reg = self._makeOne()
         reg.register(IFoo, (NS, NAME), _factory)
         self.assertEqual(len(reg._registry), 1)
@@ -325,9 +359,9 @@ class ConfigurationAdapterRegistryTests(unittest.TestCase):
         NS = 'http://namespace.example.com/'
         NAME = 'testing'
         def _factory():
-            pass
+            raise AssertionError("should not be called")
         def _rival():
-            pass
+            raise AssertionError("should not be called")
         reg = self._makeOne()
         reg.register(IFoo, (NS, NAME), _factory)
         reg.register(IFoo, (NS, NAME), _rival)
@@ -344,9 +378,9 @@ class ConfigurationAdapterRegistryTests(unittest.TestCase):
         NAME = 'testing'
         NAME2 = 'other'
         def _factory():
-            pass
+            raise AssertionError("should not be called")
         def _rival():
-            pass
+            raise AssertionError("should not be called")
         reg = self._makeOne()
         reg.register(IFoo, (NS, NAME), _factory)
         reg.register(IFoo, (NS, NAME2), _rival)
@@ -421,7 +455,7 @@ class ConfigurationAdapterRegistryTests(unittest.TestCase):
         NAME = 'testing'
         context = Context()
         def _factory():
-            pass
+            raise AssertionError("should not be called")
         reg = self._makeOne()
         reg.register(IFoo, (NS, NAME), _factory)
         self.assertEqual(reg.factory(context, (NS, NAME)), _factory)
@@ -438,7 +472,7 @@ class ConfigurationAdapterRegistryTests(unittest.TestCase):
         NAME = 'testing'
         context = Context()
         def _factory():
-            pass
+            raise AssertionError("should not be called")
         reg = self._makeOne()
         reg.register(IFoo, NAME, _factory)
         self.assertEqual(reg.factory(context, (NS, NAME)), _factory)
@@ -452,7 +486,7 @@ class ConfigurationAdapterRegistryTests(unittest.TestCase):
         NAME = 'testing'
         context = object() # doesn't provide IFoo
         def _factory():
-            pass
+            raise AssertionError("should not be called")
         reg = self._makeOne()
         reg.register(IFoo, (NS, NAME), _factory)
         with self.assertRaises(ConfigurationError):
@@ -461,6 +495,12 @@ class ConfigurationAdapterRegistryTests(unittest.TestCase):
 
 class _ConformsToIConfigurationContext(object):
 
+    def _getTargetClass(self):
+        raise NotImplementedError
+
+    def _makeOne(self, *args, **kwargs):
+        raise NotImplementedError
+
     def test_class_conforms_to_IConfigurationContext(self):
         from zope.interface.verify import verifyClass
         from zope.configuration.interfaces import IConfigurationContext
@@ -510,7 +550,7 @@ class ConfigurationMachineTests(_ConformsToIConfigurationContext,
         NS = 'http://namespace.example.com/'
         NAME = 'testing'
         def _factory(context, data, info):
-            pass
+            raise AssertionError("should not be called")
         cm = self._makeOne()
         cm.register(IConfigurationContext, (NS, NAME), _factory)
         with self.assertRaises(TypeError):
@@ -525,7 +565,7 @@ class ConfigurationMachineTests(_ConformsToIConfigurationContext,
         class IUsedIn(Interface):
             pass
         def _handler(*args, **kw):
-            pass
+            raise AssertionError("should not be called")
         NS = 'http://namespace.example.com/'
         NAME = 'testing'
         _called_with = []
@@ -535,22 +575,24 @@ class ConfigurationMachineTests(_ConformsToIConfigurationContext,
             return item
         cm = self._makeOne()
         cm.register(IConfigurationContext, (NS, NAME), _factory)
-        cm.begin((NS, NAME), {'name': 'testing',
-                                  'schema': ISchema,
-                                  'usedIn': IUsedIn,
-                                  'handler': _handler,
-                                 }, 'INFO')
+        cm.begin((NS, NAME), {
+            'name': 'testing',
+            'schema': ISchema,
+            'usedIn': IUsedIn,
+            'handler': _handler,
+        }, 'INFO')
         self.assertEqual(len(cm.stack), 2)
         root = cm.stack[0]
         self.assertIsInstance(root, RootStackItem)
         nested = cm.stack[1]
         self.assertTrue(nested is item)
         self.assertEqual(_called_with,
-                        [(cm, {'name': 'testing',
-                               'schema': ISchema,
-                               'usedIn': IUsedIn,
-                               'handler': _handler,
-                              }, 'INFO')])
+                         [(cm, {
+                             'name': 'testing',
+                             'schema': ISchema,
+                             'usedIn': IUsedIn,
+                             'handler': _handler,
+                         }, 'INFO')])
 
     def test_begin_wo___data_w_kw(self):
         from zope.interface import Interface
@@ -561,7 +603,7 @@ class ConfigurationMachineTests(_ConformsToIConfigurationContext,
         class IUsedIn(Interface):
             pass
         def _handler(*args, **kw):
-            pass
+            raise AssertionError("should not be called")
         NS = 'http://namespace.example.com/'
         NAME = 'testing'
         _called_with = []
@@ -571,23 +613,25 @@ class ConfigurationMachineTests(_ConformsToIConfigurationContext,
             return item
         cm = self._makeOne()
         cm.register(IConfigurationContext, (NS, NAME), _factory)
-        cm.begin((NS, NAME), None, 'INFO',
-                  name='testing',
-                  schema=ISchema,
-                  usedIn=IUsedIn,
-                  handler=_handler,
-                )
+        cm.begin(
+            (NS, NAME), None, 'INFO',
+            name='testing',
+            schema=ISchema,
+            usedIn=IUsedIn,
+            handler=_handler,
+        )
         self.assertEqual(len(cm.stack), 2)
         root = cm.stack[0]
         self.assertIsInstance(root, RootStackItem)
         nested = cm.stack[1]
         self.assertTrue(nested is item)
         self.assertEqual(_called_with,
-                        [(cm, {'name': 'testing',
-                               'schema': ISchema,
-                               'usedIn': IUsedIn,
-                               'handler': _handler,
-                              }, 'INFO')])
+                         [(cm, {
+                             'name': 'testing',
+                             'schema': ISchema,
+                             'usedIn': IUsedIn,
+                             'handler': _handler,
+                         }, 'INFO')])
 
     def test_end(self):
         from zope.configuration.config import RootStackItem
@@ -617,7 +661,7 @@ class ConfigurationMachineTests(_ConformsToIConfigurationContext,
             def finish(self):
                 self._finished = True
         def _handler(*args, **kw):
-            pass
+            raise AssertionError("should not be called")
         NS = 'http://namespace.example.com/'
         NAME = 'testing'
         _called_with = []
@@ -627,21 +671,23 @@ class ConfigurationMachineTests(_ConformsToIConfigurationContext,
             return item
         cm = self._makeOne()
         cm.register(IConfigurationContext, (NS, NAME), _factory)
-        cm((NS, NAME), 'INFO',
+        cm(
+            (NS, NAME), 'INFO',
             name='testing',
             schema=ISchema,
             usedIn=IUsedIn,
             handler=_handler,
-           )
+        )
         self.assertEqual(len(cm.stack), 1)
         root = cm.stack[0]
         self.assertIsInstance(root, RootStackItem)
         self.assertEqual(_called_with,
-                        [(cm, {'name': 'testing',
-                               'schema': ISchema,
-                               'usedIn': IUsedIn,
-                               'handler': _handler,
-                              }, 'INFO')])
+                         [(cm, {
+                             'name': 'testing',
+                             'schema': ISchema,
+                             'usedIn': IUsedIn,
+                             'handler': _handler,
+                         }, 'INFO')])
         self.assertTrue(item._finished)
 
     def test_getInfo_only_root_default(self):
@@ -695,12 +741,12 @@ class ConfigurationMachineTests(_ConformsToIConfigurationContext,
         cm.action(None, _a1, ('b', 2), {'foo': 'qux'}, 2)
         cm.execute_actions()
         self.assertEqual(_called_with['_a1'],
-                        [(('b', 2), {'foo': 'qux'}),
-                         (('a', 0), {'foo': 'bar'}),
-                        ])
+                         [(('b', 2), {'foo': 'qux'}),
+                          (('a', 0), {'foo': 'bar'}),
+                         ])
         self.assertEqual(_called_with['_a2'],
-                        [(('a', 1), {'foo': 'baz'}),
-                        ])
+                         [(('a', 1), {'foo': 'baz'}),
+                         ])
 
     def test_execute_actions_w_errors_w_testing(self):
         def _err(*args, **kw):
@@ -720,32 +766,66 @@ class ConfigurationMachineTests(_ConformsToIConfigurationContext,
         cm.action(None, _err)
         with self.assertRaises(ConfigurationExecutionError) as exc:
             cm.execute_actions()
-        self.assertTrue(exc.exception.etype is ValueError)
+        self.assertIs(exc.exception.etype, ValueError)
         self.assertEqual(str(exc.exception.evalue), "XXX")
         self.assertEqual(exc.exception.info, "INFO")
 
+    def _check_execute_actions_w_errors_wo_testing(self, ex_kind, cm=None):
+        ex = ex_kind('XXX')
+        def _err(*args, **kw):
+            raise ex
+        cm = cm if cm is not None else self._makeOne()
+        cm.info = 'INFO'
+        cm.action(None, _err)
+        with self.assertRaises(ex_kind) as exc:
+            cm.execute_actions()
+
+        self.assertIs(exc.exception, ex)
+
+    def test_execute_actions_w_errors_wo_testing_SystemExit(self):
+        # It gets passed through as-is
+        self._check_execute_actions_w_errors_wo_testing(SystemExit)
+
+    def test_execute_actions_w_errors_wo_testing_KeyboardInterrupt(self):
+        # It gets passed through as-is
+        self._check_execute_actions_w_errors_wo_testing(KeyboardInterrupt)
+
+    def test_execute_actions_w_errors_wo_testing_BaseException(self):
+        # It gets passed through as-is
+        class Bex(BaseException):
+            pass
+        self._check_execute_actions_w_errors_wo_testing(Bex)
+
+    def test_execute_actions_w_errors_custom(self):
+        # It gets passed through as-is, if we ask it to
+        class Ex(Exception):
+            pass
+        cm = self._makeOne()
+        cm.pass_through_exceptions += (Ex,)
+        self._check_execute_actions_w_errors_wo_testing(Ex, cm)
+
     def test_keyword_handling(self):
         # This is really an integraiton test.
         from zope.configuration.config import metans
         from zope.configuration.tests.directives import f
-        from zope.configuration._compat import b
-        from zope.configuration._compat import u
         machine = self._makeOne()
         ns = "http://www.zope.org/testing"
 
         #Register some test directives, starting with a grouping directive
         # that sets a package:
 
-        machine((metans, "groupingDirective"),
-                 name="package", namespace=ns,
-                 schema="zope.configuration.tests.directives.IPackaged",
-                 handler="zope.configuration.tests.directives.Packaged",
-                )
+        machine(
+            (metans, "groupingDirective"),
+            name="package", namespace=ns,
+            schema="zope.configuration.tests.directives.IPackaged",
+            handler="zope.configuration.tests.directives.Packaged",
+        )
 
         # set the package:
-        machine.begin((ns, "package"),
-                       package="zope.configuration.tests.directives",
-                      )
+        machine.begin(
+            (ns, "package"),
+            package="zope.configuration.tests.directives",
+        )
 
         #Which makes it easier to define the other directives:
         machine((metans, "directive"),
@@ -753,13 +833,13 @@ class ConfigurationMachineTests(_ConformsToIConfigurationContext,
                 schema=".Ik", handler=".k")
 
         machine((ns, "k"), "yee ha",
-                **{"for": u("f"), "class": u("c"), "x": u("x")})
+                **{"for": u"f", "class": u"c", "x": u"x"})
 
         self.assertEqual(len(machine.actions), 1)
         self.assertEqual(machine.actions[0],
-                         {'args': (b('f'), b('c'), b('x')),
+                         {'args': ('f', 'c', 'x'),
                           'callable': f,
-                          'discriminator': ('k', b('f')),
+                          'discriminator': ('k', 'f'),
                           'includepath': (),
                           'info': 'yee ha',
                           'kw': {},
@@ -769,6 +849,12 @@ class ConfigurationMachineTests(_ConformsToIConfigurationContext,
 
 class _ConformsToIStackItem(object):
 
+    def _getTargetClass(self):
+        raise NotImplementedError
+
+    def _makeOne(self, *args, **kwargs):
+        raise NotImplementedError
+
     def test_class_conforms_to_IStackItem(self):
         from zope.interface.verify import verifyClass
         from zope.configuration.config import IStackItem
@@ -796,7 +882,7 @@ class SimpleStackItemTests(_ConformsToIStackItem,
             context = FauxContext()
         if handler is None:
             def handler():
-                pass
+                raise AssertionError("should not be called")
         if info is None:
             info = 'INFO'
         if schema is None:
@@ -812,7 +898,7 @@ class SimpleStackItemTests(_ConformsToIStackItem,
             pass
         context = FauxContext()
         def _handler():
-            pass
+            raise AssertionError("should not be called")
         _data = {}
         ssi = self._makeOne(context, _handler, 'INFO', ISchema, _data)
         self.assertIsInstance(ssi.context, GroupingContextDecorator)
@@ -847,7 +933,7 @@ class SimpleStackItemTests(_ConformsToIStackItem,
             name = Text(required=True)
         context = FauxContext()
         def _action(context, **kw):
-            pass
+            raise AssertionError("should not be called")
         def _handler(context, **kw):
             return [(None, _action)]
         _data = {'name': 'NAME'}
@@ -870,7 +956,7 @@ class SimpleStackItemTests(_ConformsToIStackItem,
             name = Text(required=True)
         context = FauxContext()
         def _action(context, **kw):
-            pass
+            raise AssertionError("should not be called")
         def _handler(context, **kw):
             return [{'discriminator': None, 'callable': _action}]
         _data = {'name': 'NAME'}
@@ -905,7 +991,7 @@ class RootStackItemTests(_ConformsToIStackItem,
         from zope.configuration.exceptions import ConfigurationError
         class _Context(object):
             def factory(self, context, name):
-                pass
+                "does nothing"
         rsi = self._makeOne(_Context())
         with self.assertRaises(ConfigurationError):
             rsi.contained(('ns', 'name'), {}, '')
@@ -950,7 +1036,7 @@ class GroupingStackItemTests(_ConformsToIStackItem,
             _called_with.append((context, data, info))
             return _adapter
         def _action(*args, **kw):
-            pass
+            raise AssertionError("should not be called")
         class _Context(FauxContext):
             def factory(self, context, name):
                 return _factory
@@ -983,9 +1069,9 @@ class GroupingStackItemTests(_ConformsToIStackItem,
             _called_with.append((context, data, info))
             return _adapter
         def _before(*args, **kw):
-            pass
+            raise AssertionError("should not be called")
         def _after(*args, **kw):
-            pass
+            raise AssertionError("should not be called")
         class _Context(FauxContext):
             def factory(self, context, name):
                 return _factory
@@ -1012,9 +1098,9 @@ class GroupingStackItemTests(_ConformsToIStackItem,
 
     def test_finish_calls_before_if_not_already_called(self):
         def _before(*args, **kw):
-            pass
+            raise AssertionError("should not be called")
         def _after(*args, **kw):
-            pass
+            raise AssertionError("should not be called")
         class _Context(FauxContext):
             def before(self):
                 return [(None, _before)]
@@ -1022,7 +1108,7 @@ class GroupingStackItemTests(_ConformsToIStackItem,
                 return [(None, _after)]
         context = _Context()
         rsi = self._makeOne(context)
-        adapter = rsi.finish()
+        rsi.finish()
         self.assertEqual(len(context.actions), 2)
         self.assertEqual(context.actions[0], # no GSI to add extras
                          {'discriminator': None,
@@ -1107,7 +1193,7 @@ class ComplexStackItemTests(_ConformsToIStackItem,
             pass
         class WithName(object):
             def testing(self, *args):
-                pass
+                raise AssertionError("should not be called")
         meta = self._makeMeta()
         wn = meta._handler = WithName()
         meta[NAME] = (ISubSchema, 'SUBINFO')
@@ -1154,7 +1240,7 @@ class ComplexStackItemTests(_ConformsToIStackItem,
 
     def test_finish_handler_returns_oldstyle_actions(self):
         def _action():
-            pass
+            raise AssertionError("should not be called")
         def _handler():
             return [(None, _action)]
         meta = self._makeMeta()
@@ -1176,7 +1262,7 @@ class ComplexStackItemTests(_ConformsToIStackItem,
 
     def test_finish_handler_returns_newstyle_actions(self):
         def _action():
-            pass
+            raise AssertionError("should not be called")
         def _handler():
             return [{'discriminator': None, 'callable': _action}]
         meta = self._makeMeta()
@@ -1199,6 +1285,12 @@ class ComplexStackItemTests(_ConformsToIStackItem,
 
 class _ConformsToIGroupingContext(object):
 
+    def _getTargetClass(self):
+        raise NotImplementedError
+
+    def _makeOne(self, *args, **kwargs):
+        raise NotImplementedError
+
     def test_class_conforms_to_IGroupingContext(self):
         from zope.interface.verify import verifyClass
         from zope.configuration.interfaces import IGroupingContext
@@ -1256,6 +1348,12 @@ class GroupingContextDecoratorTests(_ConformsToIConfigurationContext,
 
 class _ConformsToIDirectivesContext(object):
 
+    def _getTargetClass(self):
+        raise NotImplementedError
+
+    def _makeOne(self, *args, **kwargs):
+        raise NotImplementedError
+
     def test_class_conforms_to_IDirectivesContext(self):
         from zope.interface.verify import verifyClass
         from zope.configuration.config import IDirectivesContext
@@ -1293,6 +1391,7 @@ class Test_defineSimpleDirective(unittest.TestCase):
     def _makeContext(self):
         class _Context(FauxContext):
             def __init__(self):
+                FauxContext.__init__(self)
                 self._registered = []
                 self._documented = []
             def register(self, usedIn, name, factory):
@@ -1312,7 +1411,7 @@ class Test_defineSimpleDirective(unittest.TestCase):
         context.namespace = NS
         context.info = 'INFO'
         def _handler():
-            pass
+            raise AssertionError("should not be called")
 
         self._callFUT(context, NAME, ISchema, _handler)
 
@@ -1342,7 +1441,7 @@ class Test_defineSimpleDirective(unittest.TestCase):
         context.namespace = NS
         context.info = 'INFO'
         def _handler():
-            pass
+            raise AssertionError("should not be called")
 
         self._callFUT(context, NAME, ISchema, _handler,
                       namespace='*', usedIn=IUsedIn)
@@ -1371,6 +1470,7 @@ class Test_defineGroupingDirective(unittest.TestCase):
     def _makeContext(self):
         class _Context(FauxContext):
             def __init__(self):
+                FauxContext.__init__(self)
                 self._registered = []
                 self._documented = []
             def register(self, usedIn, name, factory):
@@ -1450,6 +1550,12 @@ class Test_defineGroupingDirective(unittest.TestCase):
 
 class _ConformsToIComplexDirectiveContext(object):
 
+    def _getTargetClass(self):
+        raise NotImplementedError
+
+    def _makeOne(self, *args, **kwargs):
+        raise NotImplementedError
+
     def test_class_conforms_to_IComplexDirectiveContext(self):
         from zope.interface.verify import verifyClass
         from zope.configuration.config import IComplexDirectiveContext
@@ -1528,7 +1634,7 @@ class ComplexDirectiveDefinitionTests(_ConformsToIComplexDirectiveContext,
         self.assertEqual(_handled, [(csi.context, {'arg': 'val'})])
 
         self.assertEqual(_documented,
-                        [((NS, NAME), ISchema, IUsedIn, _handler, 'INFO')])
+                         [((NS, NAME), ISchema, IUsedIn, _handler, 'INFO')])
 
 
 class Test_subdirective(unittest.TestCase):
@@ -1688,9 +1794,8 @@ class Test_toargs(unittest.TestCase):
     def test_w_field_missing_but_default(self):
         from zope.interface import Interface
         from zope.schema import Text
-        from zope.configuration._compat import u
         class ISchema(Interface):
-            w_default = Text(default=u('default'))
+            w_default = Text(default=u'default')
         context = FauxContext()
         self.assertEqual(self._callFUT(context, ISchema, {}),
                          {'w_default': 'default'})
@@ -1727,7 +1832,7 @@ class Test_expand_action(unittest.TestCase):
 
     def test_explicit_no_extra(self):
         def _callable():
-            pass
+            raise AssertionError("should not be called")
         self.assertEqual(self._callFUT(('a', 1, None),
                                        _callable, ('b', 2), {'c': None},
                                        ('p', 'q/r'), 'INFO', 42,
@@ -1743,7 +1848,7 @@ class Test_expand_action(unittest.TestCase):
 
     def test_explicit_w_extra(self):
         def _callable():
-            pass
+            raise AssertionError("should not be called")
         self.assertEqual(self._callFUT(('a', 1, None),
                                        _callable, ('b', 2), {'c': None},
                                        ('p', 'q/r'), 'INFO', 42,
@@ -1772,27 +1877,32 @@ class Test_resolveConflicts(unittest.TestCase):
 
     def test_expands_oldstyle_actions(self):
         def _callable():
-            pass
-        self.assertEqual(self._callFUT([(None, _callable)]),
-                                       [{'discriminator': None,
-                                         'callable': _callable,
-                                         'args': (),
-                                         'kw': {},
-                                         'includepath': (),
-                                         'info': None,
-                                         'order': 0,
-                                        }])
+            raise AssertionError("should not be called")
+        self.assertEqual(
+            self._callFUT([(None, _callable)]),
+            [
+                {
+                    'discriminator': None,
+                    'callable': _callable,
+                    'args': (),
+                    'kw': {},
+                    'includepath': (),
+                    'info': None,
+                    'order': 0,
+                }
+            ]
+        )
 
     def test_wo_discriminator_clash(self):
         from zope.configuration.config import expand_action
         def _a():
-            pass
+            raise AssertionError("should not be called")
         def _b():
-            pass
+            raise AssertionError("should not be called")
         def _c():
-            pass
+            raise AssertionError("should not be called")
         def _d():
-            pass
+            raise AssertionError("should not be called")
         actions = [
             expand_action(('a', 1), _a, order=3),
             expand_action(('b', 2), _b, order=1),
@@ -1805,9 +1915,9 @@ class Test_resolveConflicts(unittest.TestCase):
     def test_w_resolvable_discriminator_clash(self):
         from zope.configuration.config import expand_action
         def _a():
-            pass
+            raise AssertionError("should not be called")
         def _b():
-            pass
+            raise AssertionError("should not be called")
         actions = [
             expand_action(('a', 1), _a, includepath=('a',)),
             expand_action(('a', 1), _b, includepath=('a', 'b')),
@@ -1819,11 +1929,11 @@ class Test_resolveConflicts(unittest.TestCase):
         from zope.configuration.config import ConfigurationConflictError
         from zope.configuration.config import expand_action
         def _a():
-            pass
+            raise AssertionError("should not be called")
         def _b():
-            pass
+            raise AssertionError("should not be called")
         actions = [
-            expand_action(('a', 1), _a, includepath=('b','c'), info='X'),
+            expand_action(('a', 1), _a, includepath=('b', 'c'), info='X'),
             expand_action(('a', 1), _b, includepath=('a',), info='Y'),
         ]
         with self.assertRaises(ConfigurationConflictError) as exc:
@@ -1834,9 +1944,9 @@ class Test_resolveConflicts(unittest.TestCase):
         from zope.configuration.config import ConfigurationConflictError
         from zope.configuration.config import expand_action
         def _a():
-            pass
+            raise AssertionError("should not be called")
         def _b():
-            pass
+            raise AssertionError("should not be called")
         actions = [
             expand_action(('a', 1), _a, includepath=('a',), info='X'),
             expand_action(('a', 1), _b, includepath=('a',), info='Y'),
@@ -1849,9 +1959,9 @@ class Test_resolveConflicts(unittest.TestCase):
         from zope.configuration.config import ConfigurationConflictError
         from zope.configuration.config import expand_action
         def _a():
-            pass
+            raise AssertionError("should not be called")
         def _b():
-            pass
+            raise AssertionError("should not be called")
         actions = [
             expand_action(('a', 1), _a, includepath=('a',), info='conflict!'),
             expand_action(
@@ -1867,13 +1977,13 @@ class Test_resolveConflicts(unittest.TestCase):
     def test_wo_discriminators_final_sorting_order(self):
         from zope.configuration.config import expand_action
         def _a():
-            pass
+            raise AssertionError("should not be called")
         def _b():
-            pass
+            raise AssertionError("should not be called")
         def _c():
-            pass
+            raise AssertionError("should not be called")
         def _d():
-            pass
+            raise AssertionError("should not be called")
         actions = [
             expand_action(None, _a, order=3),
             expand_action(None, _b, order=1),
@@ -1889,24 +1999,3 @@ class FauxContext(object):
         self.actions = []
     def action(self, **kw):
         self.actions.append(kw)
-
-def test_suite():
-    return unittest.TestSuite((
-        unittest.makeSuite(ConfigurationContextTests),
-        unittest.makeSuite(ConfigurationAdapterRegistryTests),
-        unittest.makeSuite(ConfigurationMachineTests),
-        unittest.makeSuite(SimpleStackItemTests),
-        unittest.makeSuite(RootStackItemTests),
-        unittest.makeSuite(GroupingStackItemTests),
-        unittest.makeSuite(ComplexStackItemTests),
-        unittest.makeSuite(GroupingContextDecoratorTests),
-        unittest.makeSuite(DirectivesHandlerTests),
-        unittest.makeSuite(Test_defineSimpleDirective),
-        unittest.makeSuite(Test_defineGroupingDirective),
-        unittest.makeSuite(ComplexDirectiveDefinitionTests),
-        unittest.makeSuite(Test_subdirective),
-        unittest.makeSuite(Test_provides),
-        unittest.makeSuite(Test_toargs),
-        unittest.makeSuite(Test_expand_action),
-        unittest.makeSuite(Test_resolveConflicts),
-    ))
